! ===============================================================================================
MODULE ACTPol_d156 
! ===============================================================================================
  use Options
  use Fg_subroutines
  use Foregrounds_loading

  implicit none
  logical :: initialise_act=.true.

  character(len=*), parameter, public :: actpol_likelihood_version='ACTPol_d156_like_v1'

  real(8) :: b_dat(nbin), b_sig(nbin), bval(nbin), covmat(nbin,nbin), covnew(nbin,nbin)
  real(8), dimension(:,:), allocatable :: fisher
  real(8), dimension(:,:), allocatable :: win_func_tt,win_func_te,win_func_ee
  real(8), dimension(:), allocatable :: diff_vec

  PRIVATE
  public :: actpol_d156_likelihood_init
  public :: actpol_d156_likelihood_compute
  public :: get_covmat
contains
  
! ===============================================================================================
  SUBROUTINE actpol_d156_likelihood_init
! -----------------------------------------------------------------------------------------------
    
  implicit none
  integer :: i,j,lun,il,info
  character(len=240) :: specfilename(nspec), bblfilename(nspec)
  real(8) :: covmat_new(nbin,nbin)
  logical :: good
    
  print *, 'Initializing ACTPol combined likelihood, version '//actpol_likelihood_version
  !-----------------------------------------------
  ! set file names
  !-----------------------------------------------
  specfilename(1) = trim(data_dir)//trim(ACT_data_dir)//'spectrum_TT.dat'
  specfilename(2) = trim(data_dir)//trim(ACT_data_dir)//'spectrum_TE.dat'
  specfilename(3) = trim(data_dir)//trim(ACT_data_dir)//'spectrum_EE.dat'
  bblfilename(1)  = trim(data_dir)//trim(ACT_data_dir)//'BblMean.dat'
  bblfilename(2)  = trim(data_dir)//trim(ACT_data_dir)//'BblMean_Pol.dat'
  bblfilename(3)  = trim(data_dir)//trim(ACT_data_dir)//'BblMean_Pol.dat'

  !-----------------------------------------------
  ! load spectrum and band power window functions 
  !-----------------------------------------------
    
  do j=1,nspec
     inquire(file=specfilename(j), exist=good)
     if(.not.good)then
       write(*,*) 'cant find', trim(specfilename(j)), trim(data_dir)
       stop
     endif
  
     call get_free_lun(lun)
     open(unit=lun,file=specfilename(j),form='formatted',status='unknown',action='read')  
     if (j == 1) then
        do i=1,nbintt !read TT
           read(lun,*) bval(i),b_dat(i),b_sig(i)
        enddo
     else if (j == 2) then
        do i=nbintt+1,nbintt+nbinte !read TE
           read(lun,*) bval(i),b_dat(i),b_sig(i)
        enddo
     else if (j == 3) then
        do i=nbintt+nbinte+1,nbintt+nbinte+nbinee !read EE
           read(lun,*) bval(i),b_dat(i),b_sig(i)
        enddo
     end if
     close(lun)

     inquire (file=bblfilename(j), exist=good)
     if(.not.good)then
       write(*,*) 'cant find', trim(bblfilename(j)), trim(ACT_data_dir)
       stop
     endif
 
     call get_free_lun(lun)
     open(unit=lun,file=bblfilename(j),form='formatted',status='unknown',action='read')
     if (j == 1) then
        allocate(win_func_tt(1:bmax0,1:lmax_win)) !Defined over ACTPol's full ell range 
        do i=1,bmax0
           read(lun,*) (win_func_tt(i,il), il=2,lmax_win)
        enddo
     else if (j == 2) then
        allocate(win_func_te(1:bmax0,1:lmax_win))
        do i=1,bmax0
           read(lun,*) (win_func_te(i,il), il=2,lmax_win)
        enddo
     else if (j == 3) then
        allocate(win_func_ee(1:bmax0,1:lmax_win))
        do i=1,bmax0
           read(lun,*) (win_func_ee(i,il), il=2,lmax_win)
        enddo
     end if
     close(lun)
   enddo


   !------------------------------------------------- 
   !Read covariance matrix 
   !-------------------------------------------------

    call get_covmat(covmat_new)
    do i=1,nbin
       covmat(i,1:nbin) = covmat_new(i,1:nbin)
    enddo

    initialise_act = .false.

  END SUBROUTINE actpol_d156_likelihood_init
! ===============================================================================================

! ===============================================================================================
  SUBROUTINE get_covmat(covmat_new)
! ===============================================================================================
          
  implicit none 
      
  integer :: i,j,k,l,lun,n,stat
  character(len=240) :: binfilename(nspec), covfilename
  logical :: good
  real(8) :: covmat(nbin,nbin)
  real(8), intent(out) :: covmat_new(nbin,nbin)
  integer :: lmin(nspec),lmax(nspec)
  integer :: blmean(nspec,bmax0),blmin(nspec,bmax0),blmax(nspec,bmax0),bmin(nspec),bmax(nspec)
       
  lmin(1)=lmintt
  lmax(1)=lmaxtt
  lmin(2)=lminte
  lmax(2)=lmaxte
  lmin(3)=lminee
  lmax(3)=lmaxee

 !------------------------------------------------- 
 !Read covariance matrixes 
 !-------------------------------------------------
 
  covfilename    = trim(data_dir)//trim(ACT_data_dir)//'CovMat_Sim.dat'
  binfilename(1) = trim(data_dir)//trim(ACT_data_dir)//'Binning.dat'
  binfilename(2) = trim(data_dir)//trim(ACT_data_dir)//'Binning.dat'
  binfilename(3) = trim(data_dir)//trim(ACT_data_dir)//'Binning.dat'
     
  call get_free_lun( lun )
  open(unit=lun,file=covfilename,form='formatted',status='unknown',action='read')
  do i=1,nbin
     read(lun,*) covmat(i,1:nbin)
  enddo
  close(lun)
   
  !------------------------------------------------- 
  !Get ell ranges
  !-------------------------------------------------
  
  do j=1,nspec
     inquire(file=binfilename(j), exist=good)
     if(.not.good)then
       write(*,*) 'cant find', trim(binfilename(j)), trim(ACT_data_dir)
       stop
     endif

     call get_free_lun(lun)
     open(unit=lun,file=binfilename(j),form='formatted',status='unknown',action='read')
     do i=1,bmax0
        read(lun,*) blmin(j,i), blmax(j,i), blmean(j,i)
     enddo
     close(lun)
     do i=1,bmax0
        if(lmin(j) .gt. blmin(j,i)) bmin(j)=i+1-b1+1
        if(lmin(j) == blmin(j,i))   bmin(j)=i-b1+1
        if(blmean(j,i) .le. lmax(j)) bmax(j)=i-b1+1
     enddo
  enddo


  !------------------------------------------------- 
  !Change covmat to select the ell range
  !-------------------------------------------------
   covmat_new(:,:) = 0.d0
   do i=1,nbin
      covmat_new(i,i) = 1d+10
   enddo

   ! First row, TT x TT, TE, EE
   covmat_new(bmin(1):bmax(1),bmin(1):bmax(1)) = covmat(bmin(1):bmax(1),bmin(1):bmax(1))
   covmat_new(bmin(1):bmax(1),nbintt+bmin(2):nbintt+bmax(2)) = covmat(bmin(1):bmax(1),nbintt+bmin(2):nbintt+bmax(2))
   covmat_new(bmin(1):bmax(1),nbintt+nbinte+bmin(3):nbintt+nbinte+bmax(3)) = covmat(bmin(1):bmax(1),nbintt+nbinte+bmin(3):nbintt+nbinte+bmax(3))

   ! Second row, TE x TT, TE, EE
   covmat_new(nbintt+bmin(2):nbintt+bmax(2),bmin(1):bmax(1)) = covmat(nbintt+bmin(2):nbintt+bmax(2),bmin(1):bmax(1))
   covmat_new(nbintt+bmin(2):nbintt+bmax(2),nbintt+bmin(2):nbintt+bmax(2)) = covmat(nbintt+bmin(2):nbintt+bmax(2),nbintt+bmin(2):nbintt+bmax(2))
   covmat_new(nbintt+bmin(2):nbintt+bmax(2),nbintt+nbinte+bmin(3):nbintt+nbinte+bmax(3)) = covmat(nbintt+bmin(2):nbintt+bmax(2),nbintt+nbinte+bmin(3):nbintt+nbinte+bmax(3))

   ! Third row, EE x TT, TE, EE
   covmat_new(nbintt+nbinte+bmin(3):nbintt+nbinte+bmax(3),bmin(1):bmax(1)) = covmat(nbintt+nbinte+bmin(3):nbintt+nbinte+bmax(3),bmin(1):bmax(1))
   covmat_new(nbintt+nbinte+bmin(3):nbintt+nbinte+bmax(3),nbintt+bmin(2):nbintt+bmax(2)) = covmat(nbintt+nbinte+bmin(3):nbintt+nbinte+bmax(3),nbintt+bmin(2):nbintt+bmax(2))
   covmat_new(nbintt+nbinte+bmin(3):nbintt+nbinte+bmax(3),nbintt+nbinte+bmin(3):nbintt+nbinte+bmax(3)) = covmat(nbintt+nbinte+bmin(3):nbintt+nbinte+bmax(3),nbintt+nbinte+bmin(3):nbintt+nbinte+bmax(3))

   END SUBROUTINE get_covmat
  !==============================================================================================


! ===============================================================================================
  SUBROUTINE actpol_d156_likelihood_compute(cl,a_tsz,a_ksz,xi,a_d,a_s,a_c,a_g,a_tps,a_ps,calt,calp,like)
! -----------------------------------------------------------------------------------------------
    
  implicit none
  real(8), intent(in) :: cl(2:tt_lmax,1:nspec+1)
  real(8), intent(in) :: a_tsz,a_ksz,xi,a_d,a_s,a_c,a_g,a_tps,a_ps,calt,calp
  real(8), intent(out) :: like
  integer :: bin_no,lun,il,i,j,info
  real(8) :: ell(2:lmax_win),cltt(2:lmax_win),clte(2:lmax_win),clee(2:lmax_win)
  real(8) :: fgtt(2:lmax_win),fgpol(2:lmax_win),fgttpol(2:lmax_win)
  real(8) :: c_th(nbinw),diffs(nbin),b_th(nbin)
  real(8) :: dlnlike,lndet,fake
  real(8), dimension(:), allocatable :: tmp
  real(8) :: like_tt, like_pol


! Calculate foregrounds
  fgtt(2:lmax_win)=0.d0
  fgpol(2:lmax_win)=0.d0
  fgttpol(2:lmax_win)=0.d0
  do il=2,lmax_win
     fgtt(il)   = a_c*clc(il) & !clustered-cib
                  +(a_d+a_s)*clp(il) & !poisson-cib + poisson-radio 
                  +a_g*clcir(il) &!galactic dust
                  +(a_tsz+a_ksz)*clsz(il) & !tsz+ksz
                  -2.d0*sqrt(a_c*a_tsz)*xi*clszcib(il) !tszxcib

     fgpol(il) =  a_ps*clp(il) !polarized radio poisson 

     fgttpol(il) = a_tps*clp(il) !temperature/polarized radio poisson
  enddo

! Convert back CMB to Cls 
  cltt(2:lmax_win)=0.d0 
  clte(2:lmax_win)=0.d0
  clee(2:lmax_win)=0.d0
  cltt(2:tt_lmax)=cl(2:tt_lmax,2)
  clte(2:tt_lmax)=cl(2:tt_lmax,3)
  clee(2:tt_lmax)=cl(2:tt_lmax,4)
!!! Careful! this is valid for cosmomc Cls inputs, for camb Cls use 2,5,3,4

!Calculate CMB+Fg
  cltt(2:lmax_win)=cltt(2:lmax_win)+fgtt(2:lmax_win)
  clte(2:lmax_win)=clte(2:lmax_win)+fgttpol(2:lmax_win)
  clee(2:lmax_win)=clee(2:lmax_win)+fgpol(2:lmax_win)

!Multiply theory Cls by window functions to get bandpowers
   c_th(1:nbinw)=0.d0
   c_th(1:bmax0)=MATMUL(win_func_tt(1:bmax0,2:lmax_win),cltt(2:lmax_win))
   c_th(bmax0+1:2*bmax0)=MATMUL(win_func_te(1:bmax0,2:lmax_win),clte(2:lmax_win))
   c_th(2*bmax0+1:3*bmax0)=MATMUL(win_func_ee(1:bmax0,2:lmax_win),clee(2:lmax_win))

   b_th(1:nbin)=0.d0
   b_th(1:nbintt)=c_th(b1:bmax0)*2.d0*pi/dble(bval(1:nbintt)*(bval(1:nbintt)+1))*calt**2.d0
   b_th(nbintt+1:nbintt+nbinte)=c_th(bmax0+b1:2*bmax0)*2.d0*pi/dble(bval(1:nbinte)*(bval(nbintt+1:nbintt+nbinte)+1))*calt**2.d0*calp
   b_th(nbintt+nbinte+1:nbintt+nbinte+nbinee)=c_th(2*bmax0+b1:3*bmax0)*2.d0*pi/dble(bval(1:nbinee)*(bval(nbintt+nbinte+1:nbintt+nbinte+nbinee)+1))*calt**2.d0*calp**2.d0

!Basic chisq 
   like = 0.d0
   diffs = b_dat - b_th

!Only Temperature
   if((use_act_tt .eqv. .true.) .and. (use_act_pol .eqv. .false.)) then
      bin_no=nbintt
      allocate(fisher(bin_no,bin_no))
      allocate(diff_vec(bin_no),tmp(bin_no))
      diff_vec(:)=diffs(1:bin_no)
      fisher(:,:)=covmat(1:bin_no,1:bin_no)
!Only EE
   else if((use_act_ee .eqv. .true.)) then
      bin_no=nbinee
      allocate(fisher(bin_no,bin_no))
      allocate(diff_vec(bin_no),tmp(bin_no))
      diff_vec(:)=diffs(nbintt+nbinte+1:nbintt+nbinte+nbinee)
      fisher(:,:)=covmat(nbintt+nbinte+1:nbintt+nbinte+nbinee,nbintt+nbinte+1:nbintt+nbinte+nbinee)
!Only Polarization
   else if((use_act_tt .eqv. .false.) .and. (use_act_pol .eqv. .true.)) then
      bin_no=nbinte+nbinee
      allocate(fisher(bin_no,bin_no))
      allocate(diff_vec(bin_no),tmp(bin_no))
      diff_vec(:)=diffs(nbintt+1:nbin)
      fisher(:,:)=covmat(nbintt+1:nbin,nbintt+1:nbin)
!Full ACTPol
   else if ((use_act_tt .eqv. .true.) .and. (use_act_pol .eqv. .true.)) then
      bin_no=nbin
      allocate(fisher(nbin,nbin))
      allocate(diff_vec(nbin),tmp(nbin))
      diff_vec(:)=diffs(:)
      fisher(:,:)=covmat(:,:)
   else
      write(*,*) 'Fail: no options chosen'
   endif
    
!Invert covmat
   call dpotrf('U',bin_no,fisher,bin_no,info)
   if(info.ne.0)then
      print*, ' info in dpotrf =', info
      stop
   endif
   call dpotri('U',bin_no,fisher,bin_no,info)
   if(info.ne.0)then
      print*, ' info in dpotri =', info
      stop
   endif
   do i=1,bin_no
      do j=i,bin_no
         fisher(j,i)=fisher(i,j)
      enddo
   enddo
 
   tmp=matmul(fisher,diff_vec)
   dlnlike=sum(tmp*diff_vec)
   deallocate(fisher,diff_vec,tmp)
    
   like = dlnlike/2.d0
    
  end SUBROUTINE actpol_d156_likelihood_compute
! ===============================================================================================
 
END MODULE ACTPol_d156
! ===============================================================================================
