!===========================================================
program test
!===========================================================
!ACTPol_like 
!E. Calabrese, J. Dunkley 2014
!Based on WMAP likelihood codes and ACT code
!===========================================================

use Options
use Fg_subroutines
use ACTPol_like

implicit none

real(8), dimension(:,:), allocatable :: cell
real(8)            :: a_tsz,a_ksz,xi,a_d,a_s,a_c,a_g,a_tps,a_ps,calt,calp
character(len=128) :: filename
real(8)            :: like
integer            :: lun,il,i

!----------------------------------------------------------
print *,""
print *,"ACTPol likelihood test"
print *,"==================================="

!----------------------------------------------------------
! read in test Cls
!----------------------------------------------------------
filename = trim(data_dir)//'base_planck_lowl_lowLike_highL.bestfit_cl'
write(*,*)"Reading in Cls from: ",trim(filename)
write(*,*)"\033[31mCls are passed to the likelihood following cosmomc order: TT,TE,EE. Change ACTPol_d156.f90 for different input Cls.\033[0m"
call get_free_lun(lun)
allocate(cell(2:tt_lmax,1:nspec+2))
cell(:,:)=0.d0
open(unit=lun,file=filename,action='read',status='old')
do il=2,tt_lmax
   read(lun,*) (cell(il,i), i=1,nspec+2)
enddo
close(lun)

call ACTPol_likelihood_init

!----------------------------------------------------------
! nuisance parameters
a_tsz =0.1322996E+01 !tSZ power
a_ksz =0.5785652E+00 !kSZ power
xi    =0.1500305E+00 !tSZ-CIB cross-correlation coefficient
a_d   =0.8400872E+01 !CIB Poisson power 
a_s   =0.2845232E+01 !Radio Poisson power 
a_c   =0.3052210E+01 !CIB clustered power
a_g   =0.d0          !Galactic dust 
a_tps =0.3000310E-01 !TE Poisson sources
a_ps  =0.1704363E+01 !Radio polarizes Poisson sources
calt  =0.1005729E+01 !Overall temperature calibration
calp  =0.9780945E+00 !Polarization gain

call actpol_likelihood_compute(cell,a_tsz,a_ksz,xi,a_d,a_s,a_c,a_g,a_tps,a_ps,calt,calp,like)

if((use_act_tt .eqv. .true.) .and. (use_act_pol .eqv. .false.)) then
  write(*,*) '------------------------------------------------------------'
  write(*,*) 'ACT(T) -2lnL=', 2.*like
  write(*,*) ''
  write(*,*) 'Expected -2lnL =  59.6079290329251'
  write(*,*) '------------------------------------------------------------'
  write(*,*) ''
end if

if((use_act_tt .eqv. .false.) .and. (use_act_pol .eqv. .true.)) then
  write(*,*) '-------------------------------------------------------------------------'
  write(*,*) 'ACT(P) -2lnL=', 2.*like
  write(*,*) '' 
  write(*,*) 'Expected -2lnL =  122.794110397732 '
  write(*,*) '-------------------------------------------------------------------------'
  write(*,*) ''
end if

if((use_act_tt .eqv. .true.) .and. (use_act_pol .eqv. .true.)) then
  write(*,*) '-------------------------------------------------------------------------------'
  write(*,*) 'ACT(T+P) -2lnL=', 2.*like
  write(*,*) ''
  write(*,*) 'Expected -2lnL =    183.338514718240'
  write(*,*) '-------------------------------------------------------------------------------'
  write(*,*) ''
end if

if(use_act_ee .eqv. .true.) then
  write(*,*) '-------------------------------------------------------------------------------'
  write(*,*) 'ACT(EE) -2lnL=', 2.*like
  write(*,*) ''
  write(*,*) 'Expected -2lnL =  57.0416121202041'
  write(*,*) '-------------------------------------------------------------------------------'
  write(*,*) ''
end if

end program test
